// -------------------------------------------------------------
//	Title		:	MidiPP export functions
//	Copyright	:	(C)2002 HRA!
//					http://www.imasy.or.jp/~hra/
//	Revision	:	03.30.2002
// -------------------------------------------------------------

#ifndef _MIDIPP_H_
#define	_MIDIPP_H_

#ifdef __cplusplus
extern "C" {
#endif

// -------------------------------------------------------------
//	Defines
#define	MP_NUMBER_OF_ELEMENT( Array )	(sizeof(Array)/sizeof((Array)[0]))
#define	MP_GET_OFFSET( Type, Member )	( (DWORD) &(((Type*)0)->Member) )

// -------------------------------------------------------------
//	Typedefs
typedef struct tagHMPHANDLE {
	LPCTSTR		pcsSignature;
} *HMPHANDLE;

typedef HMPHANDLE	HMPSOUNDBUFFER;
typedef HMPHANDLE	HMPMIDIDEVICE;

typedef enum {
	MP_SUCCESS				= 0,
	MP_FAILURE				= 1,
	MP_NOT_ENOUGH_MEMORY	= 3,
	MP_ILLEGAL_PARAMETER	= 5,
	MP_NOT_FOUND			= 7,
	MP_UNSUPPORTED			= 9,
	MP_NOT_INITIALIZED		= 11,
	MP_INVALID_HANDLE		= 13,
	MP_BUSY					= 15,
	MP_READ					= 17,
	MP_WRITE				= 19,
} MP_ERROR_T;

#define	IS_MP_ERROR( ErrorCode )	( ((DWORD)(ErrorCode) & 1) == 1 )

typedef struct tagMP_PLUGIN_INFO {
	DWORD		StructureSize;					//	sizeof( MP_PLUGIN_INFO )
	LPCTSTR		pcsPluginName;					//	address of plugin name
	LPCTSTR		pcsVersion;						//	address of measure version
	LPCTSTR		pcsRevision;					//	address of minor version
	LPCTSTR		pcsCopyright;					//	address of copyright
	LPCTSTR		pcsURL;							//	address of internet URL
	LPCTSTR		pcsPlayableSoundFormatNames;	//	address of supported file extention (ex. "WAV")
	LPCTSTR		pcsPlayableMusicFormatNames;	//	address of supported file extention (ex. "MID;MIDI")
} MP_PLUGIN_INFO, *PMP_PLUGIN_INFO;

enum {
	MP_MONORAL				= 1,
	MP_STEREO				= 2,
};	//	Channels

enum {
	MP_SINGLE				= 1,
	MP_INFINITE_LOOP		= 0xFFFFFFFF,
};	//	PlayMode

typedef struct tagMP_SOUND_SETTING {
	DWORD		StructureSize;					//	sizeof( MP_SOUND_SETTING )
	DWORD		SamplingRate;					//	11025, 22050 or 44100
	DWORD		BitRate;						//	8 or 16
	DWORD		Channels;						//	MP_MONORAL or MP_STEREO
} MP_SOUND_SETTING, *PMP_SOUND_SETTING;

typedef struct tagMP_INTERFACE {
	DWORD			NumberOfInterfaceFunctions;
	//	v1.00 or later
	MP_ERROR_T		(WINAPI *GetInformation)( PMP_PLUGIN_INFO pPluginInfo, DWORD StructureSize );
	MP_ERROR_T		(WINAPI *IsSupportedLanguage)( DWORD LanguageId );
	MP_ERROR_T		(WINAPI *InitializeSoundDevice)( PMP_SOUND_SETTING pSoundDeviceSetting );
	MP_ERROR_T		(WINAPI *TerminateSoundDevice)( VOID );

	HMPSOUNDBUFFER	(WINAPI *CreateSoundBuffer)( DWORD NumberOfSample, DWORD Channels );

	HMPMIDIDEVICE	(WINAPI *CreateMidiDevice)( DWORD NumberOfInstrument );
	MP_ERROR_T		(WINAPI *SetMusicVolume)( HMPMIDIDEVICE hMidiDevice, DWORD Volume );	//	Volume = [0,127]
	MP_ERROR_T		(WINAPI *SetPlayEndNotify)( HMPMIDIDEVICE hMidiDevice, HWND hWindow, UINT Message );

	MP_ERROR_T		(WINAPI *LoadFile)( HMPHANDLE hSoundInstance, LPCTSTR pcsFileName );
	MP_ERROR_T		(WINAPI *Play)( HMPHANDLE hSoundInstance, DWORD PlayMode );
	MP_ERROR_T		(WINAPI *Stop)( HMPHANDLE hSoundInstance );
	MP_ERROR_T		(WINAPI *Delete)( HMPHANDLE hSoundInstance );
	MP_ERROR_T		(WINAPI *IsPlaying)( HMPHANDLE hSoundInstance );
	//	v1.01 or later
	MP_ERROR_T		(WINAPI *SendMidiMessage)( HMPMIDIDEVICE hMidiDevice, LPBYTE pMessage, DWORD NumberOfMessage );
	MP_ERROR_T		(WINAPI *SetFrequencyCallFunction)( VOID (*pCallBackFunction)( LPVOID ), LPVOID pUserInstance );
	MP_ERROR_T		(WINAPI *GetVirtualTickCount)( LPDWORD pVirtualMillisec );
	//	v1.02 or later
	MP_ERROR_T		(WINAPI *ResetMidiDevice)( HMPMIDIDEVICE hMidiDevice );
	//	v1.03 or later
	MP_ERROR_T		(WINAPI *GetWaveAddress)( HMPSOUNDBUFFER hSoundBuffer, PVOID *ppWaveAddress );
	HMPMIDIDEVICE	(WINAPI *CreateMidiDeviceEx)( DWORD NumberOfInstrument, LPCTSTR pcsFileName );
} MP_INTERFACE, *PMP_INTERFACE;

#define	MP_GET_INTERFACE_ID( Function )	\
		( (MP_GET_OFFSET( MP_INTERFACE, Function ) - sizeof(DWORD))/sizeof(LPVOID) )
#define	IS_MP_SUPPORTED_INTERFACE( pInterface, Function )	\
		( MP_GET_INTERFACE_ID(Function) < (pInterface)->NumberOfInterfaceFunction )

#define	MP_MINIMUM_INTERFACE_SIZE	56			//	by BYTE

// -------------------------------------------------------------
//	Public APIs
#define	MP_GET_INTERFACE	_T("GetInterface")
typedef MP_ERROR_T (*PGETINTERFACE)( PMP_INTERFACE pInterface, DWORD StructureSize );

#ifdef __cplusplus
}
#endif

#endif
