// -------------------------------------------------------------
//	Title		:	MidiPP Test Program
//	Copyright	:	(C)2002 HRA!
// -------------------------------------------------------------

#include <windows.h>
#include <Commdlg.h>
#include <tchar.h>
#include "MidiPP.h"
#include "Resource.h"

// -------------------------------------------------------------
static HINSTANCE hInstance;
static HWND hMainWindow;

static LPCTSTR pcsError = _T("Error");
static LPCTSTR pcsDllLoadError = _T("cannot find MidiPP.DLL");
static LPCTSTR pcsCreateDialogError = _T("cannot create dialog");
static LPCTSTR pcsGetInterfaceError = _T("uncompatible MidiPP.DLL");
static LPCTSTR pcsInitializeSoundDeviceError = _T("initialize SoundDevice error");

static LPCTSTR pcsReport = _T("Report");
static LPCTSTR pcsEndOfMusic = _T("end of play music");

//	for MidiPP
static LPCTSTR pcsMidiPP = _T("MidiPP.DLL");
static HMODULE hMidiPP;
static MP_INTERFACE MpInterface;
static HMPSOUNDBUFFER hSoundBuffer = (HMPSOUNDBUFFER) MP_INVALID_HANDLE;
static HMPMIDIDEVICE hMidiDevice = (HMPMIDIDEVICE) MP_INVALID_HANDLE;

// -------------------------------------------------------------
static BOOL LoadFileName( LPTSTR pFileName, UINT SizeByTchar, LPCTSTR pcsExtension ) {
	OPENFILENAME	OpenFileName;

	memset( pFileName, 0, SizeByTchar * sizeof(TCHAR) );
	memset( &OpenFileName, 0, sizeof(OpenFileName) );
	OpenFileName.lpstrFile	= pFileName;
	OpenFileName.hInstance	= hInstance;
	OpenFileName.Flags		= OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_PATHMUSTEXIST;
	OpenFileName.hwndOwner	= hMainWindow;
	OpenFileName.nMaxFile	= SizeByTchar;
	OpenFileName.lpstrFilter= pcsExtension;
	OpenFileName.lStructSize= sizeof(OpenFileName);
	if( !GetOpenFileName( &OpenFileName ) ) {
		return FALSE;
	}
	return TRUE;
}

// -------------------------------------------------------------
static BOOL LoadMidi( VOID ) {
	TCHAR		sFileName[ MAX_PATH ];

	if( IS_MP_ERROR( hMidiDevice ) ) {
		return FALSE;
	}
	if( !LoadFileName(sFileName, MP_NUMBER_OF_ELEMENT(sFileName), _T("*.MID\0")) ) {
		return FALSE;
	}

	MpInterface.LoadFile( hMidiDevice, sFileName );
	return TRUE;
}

// -------------------------------------------------------------
static BOOL PlayMidi( VOID ) {

	if( IS_MP_ERROR( hMidiDevice ) ) {
		return FALSE;
	}

	MpInterface.Play( hMidiDevice, MP_SINGLE );
	return TRUE;
}

// -------------------------------------------------------------
static BOOL StopMidi( VOID ) {

	if( IS_MP_ERROR( hMidiDevice ) ) {
		return FALSE;
	}

	MpInterface.Stop( hMidiDevice );
	return TRUE;
}

// -------------------------------------------------------------
static BOOL LoadSoundBuffer( VOID ) {
	TCHAR		sFileName[ MAX_PATH ];

	if( IS_MP_ERROR( hSoundBuffer ) ) {
		return FALSE;
	}
	if( !LoadFileName(sFileName, MP_NUMBER_OF_ELEMENT(sFileName), _T("*.WAV\0")) ) {
		return FALSE;
	}

	MpInterface.LoadFile( hSoundBuffer, sFileName );
	return TRUE;
}

// -------------------------------------------------------------
static BOOL PlaySoundBuffer( VOID ) {

	if( IS_MP_ERROR( hSoundBuffer ) ) {
		return FALSE;
	}

	MpInterface.Play( hSoundBuffer, MP_SINGLE );
	return TRUE;
}

// -------------------------------------------------------------
static BOOL StopSoundBuffer( VOID ) {

	if( IS_MP_ERROR( hSoundBuffer ) ) {
		return FALSE;
	}

	MpInterface.Stop( hSoundBuffer );
	return TRUE;
}

// -------------------------------------------------------------
static BOOL OnCommand( HWND hWindow, WPARAM Param1, LPARAM Param2 ) {
	BYTE SendData[3] = { 0x90, 0x00, 127 };

	switch( Param1 ) {
	case IDC_MIDI_LOAD:
		return LoadMidi();
	case IDC_MIDI_PLAY:
		return PlayMidi();
	case IDC_MIDI_STOP:
		return StopMidi();
	case IDC_WAV_LOAD:
		return LoadSoundBuffer();
	case IDC_WAV_PLAY:
		return PlaySoundBuffer();
	case IDC_WAV_STOP:
		return StopSoundBuffer();
	case IDC_NOTEON:
	case IDC_NOTEOFF:
		if( Param1 == IDC_NOTEON ) {
			SendData[0] = 0x99;
		} else {
			SendData[0] = 0x89;
		}
		SendData[1] = GetDlgItemInt( hWindow, IDC_NOTENO, NULL, FALSE ) & 0x7F;
		MpInterface.SendMidiMessage( hMidiDevice, SendData, sizeof(SendData) );
		if( Param1 == IDC_NOTEOFF ) {
			SetDlgItemInt( hWindow, IDC_NOTENO, (SendData[1] + 1) & 127, FALSE );
		}
	}
	return FALSE;
}

// -------------------------------------------------------------
static BOOL OnInitDialog( HWND hWindow ) {
	MP_PLUGIN_INFO	PluginInfo;

	if( IS_MP_ERROR(MpInterface.GetInformation( &PluginInfo, sizeof(PluginInfo) )) ) {
		return FALSE;
	}

	SetWindowText( hWindow, PluginInfo.pcsPluginName );
	SetDlgItemText( hWindow, IDC_INFO, PluginInfo.pcsCopyright );
	return TRUE;	
}

// -------------------------------------------------------------
static BOOL CALLBACK WindowEventHandler( HWND hWindow, UINT Message, 
		WPARAM Param1, LPARAM Param2 ) {

	switch( Message ) {
	case WM_INITDIALOG:
		return OnInitDialog( hWindow );
	case WM_COMMAND:
		return OnCommand( hWindow, Param1, Param2 );
	case WM_CLOSE:
		DestroyWindow( hWindow );
		return TRUE;
	case WM_DESTROY:
		PostQuitMessage( 0 );
		return TRUE;
	case WM_APP:
		MessageBox( hWindow, pcsEndOfMusic, pcsReport, MB_OK );
	}
	return FALSE;
}

// -------------------------------------------------------------
static BOOL InitializeApplication( VOID ) {
	TCHAR sFilePath[ MAX_PATH ];
	LPTSTR pEndMark;
	PGETINTERFACE pGetInterface;
	MP_SOUND_SETTING SoundSetting;

	//	Load MidiPP.DLL
	memset( sFilePath, 0, sizeof(sFilePath) );
	GetModuleFileName( NULL /* This */, sFilePath, MP_NUMBER_OF_ELEMENT(sFilePath) );
	pEndMark = _tcsrchr( sFilePath, _T('\\') );
	if( pEndMark == NULL ) {
		pEndMark = sFilePath;
	}
	else {
		++pEndMark;
	}
	_tcscpy( pEndMark, pcsMidiPP );
	hMidiPP = LoadLibrary( sFilePath );
	if( hMidiPP == NULL ) {
		MessageBox( NULL, pcsDllLoadError, pcsError, MB_OK | MB_ICONWARNING );
		return FALSE;
	}

	//	Initialize MidiPP.DLL
	pGetInterface = (PGETINTERFACE) GetProcAddress( hMidiPP, MP_GET_INTERFACE );
	if( pGetInterface == NULL ) {
		MessageBox( NULL, pcsGetInterfaceError, pcsError, MB_OK | MB_ICONWARNING );
		FreeLibrary( hMidiPP );
		return FALSE;
	}
	if( IS_MP_ERROR( pGetInterface(&MpInterface, sizeof(MpInterface)) ) ) {
		MessageBox( NULL, pcsGetInterfaceError, pcsError, MB_OK | MB_ICONWARNING );
		FreeLibrary( hMidiPP );
		return FALSE;
	}
	SoundSetting.StructureSize = sizeof(SoundSetting);
	SoundSetting.BitRate = 16;
	SoundSetting.Channels = MP_STEREO;
	SoundSetting.SamplingRate = 44100;
	if( IS_MP_ERROR( MpInterface.InitializeSoundDevice( &SoundSetting ) ) ) {
		MessageBox( NULL, pcsInitializeSoundDeviceError, pcsError, MB_OK | MB_ICONWARNING );
		FreeLibrary( hMidiPP );
		return FALSE;
	}
	hSoundBuffer = MpInterface.CreateSoundBuffer( 0, MP_MONORAL );
	hMidiDevice = MpInterface.CreateMidiDevice( 32 );

	//	DialogBox
	hMainWindow = CreateDialog( hInstance, MAKEINTRESOURCE(IDD_MAINDIALOG), 
			NULL /* no have parent */, WindowEventHandler );
	if( hMainWindow == NULL ) {
		MessageBox( NULL, pcsCreateDialogError, pcsError, MB_OK | MB_ICONWARNING );
		FreeLibrary( hMidiPP );
		return FALSE;
	}

	//	notify message
	MpInterface.SetPlayEndNotify( hMidiDevice, hMainWindow, WM_APP );

	ShowWindow( hMainWindow, SW_SHOW );
	UpdateWindow( hMainWindow );
	return TRUE;
}

// -------------------------------------------------------------
VOID TerminateApplication( VOID ) {

	//	Terminate MidiPP.DLL
	MpInterface.TerminateSoundDevice();
	FreeLibrary( hMidiPP );
}

// -------------------------------------------------------------
int WINAPI WinMain( HINSTANCE hIns, HINSTANCE hPrevIns, LPTSTR pCommand, int Show ) {
	MSG Message;

	hInstance = hIns;
	if( !InitializeApplication() ) {
		return 0;
	}

	while( GetMessage( &Message, NULL, 0, 0 ) ) {
		TranslateMessage( &Message );
		DispatchMessage( &Message );
	}

	TerminateApplication();

	return Message.wParam;
}
